"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var AdmZip = require("adm-zip");
var base64 = require("base-64");
var fs = require("fs");
var getmac = require("getmac");
var jsonfile = require("jsonfile");
var os = require("os");
var path = require("path");
var PropertiesReader = require("properties-reader");
var request = require("superagent");
var touch = require("touch");
var xmldom = require("xmldom");
var p = require("prompt-sync");
var prompt = p({ sigint: true });
exports.forbiddenFields = ["email", "password", "host", "orgs", "defaultOrgId", "defaultEnvId"];
exports.configDir = os.homedir() + "/.jbcli";
exports.logDir = exports.configDir + "/logs";
exports.credFile = exports.configDir + "/.credentials";
exports.configFile = exports.configDir + "/config.json";
exports.citizenApiPrefix = "/v1";
exports.harmonyApiPrefix = "/jitterbit-cloud-restful-service";
exports.csApiPrefix = "/design-studio/api/v1";
function authenticate(args, callback, callAgain) {
    if (!isLoggedIn()) {
        console.error("ERROR: No credentials stored. Log in first with `jbcli login`.");
        process.exit(1);
    }
    var userInfo = readCred();
    var reqURL = userInfo["host"] + exports.citizenApiPrefix + "/login";
    getmac.getMac(function (err, macAddress) {
        if (err) {
            printDetailedError("Could not get MAC address of device", err);
            process.exit(1);
        }
        request.post(reqURL)
            .set({
            browser: "N/A",
            client: "jbcli",
            os: os.type(),
        })
            .type("form")
            .send({ email: userInfo["email"], password: base64.decode(userInfo["password"]), deviceId: macAddress })
            .end(function (erro, res) {
            if (erro) {
                if (erro.response.body.errorCode === "VALIDATE_TFA_LOGIN_EMAIL") {
                    console.log("Two-factor authentication is enabled for this organization. " +
                        "Please check your email and input the code that you received from Jitterbit.");
                    var code = "";
                    try {
                        code = prompt.hide("Code: ");
                    }
                    catch (err) {
                        printDetailedError("Could not read code. This likely means that you are using Cygwin on Windows. " +
                            "If so, please use Command Prompt instead.", err);
                        process.exit(1);
                    }
                    reqURL = userInfo["host"] + exports.citizenApiPrefix + "/login/tfacode";
                    request.post(reqURL)
                        .send({
                        code: code,
                        deviceId: macAddress,
                        email: userInfo["email"],
                        password: base64.decode(userInfo["password"]),
                    })
                        .end(function (error, cres) {
                        if (error) {
                            printDetailedError("Authentication failed", erro);
                            process.exit(1);
                        }
                        else {
                            var token = cres.body.authenticationToken;
                            if (!token) {
                                console.error("ERROR: The login succeeded, but no authentication token was returned.");
                                process.exit(1);
                            }
                            callback(args, userInfo["host"], token, callAgain);
                        }
                    });
                }
                else {
                    printDetailedError("Authentication failed", erro);
                    process.exit(1);
                }
            }
            else {
                var token = res.body.authenticationToken;
                if (!token) {
                    console.error("ERROR: The login succeeded, but no authentication token was returned.");
                    process.exit(1);
                }
                callback(args, userInfo["host"], token, callAgain);
            }
        });
    });
}
exports.authenticate = authenticate;
function handleHarmonyLoginResponse(res, host, email, password) {
    var authToken = res.body.authenticationToken;
    var orgId = res.body.defaultOrgId;
    var reqURL = host + exports.harmonyApiPrefix + "/postlogin/details/" + orgId;
    request.get(reqURL)
        .set({ authToken: authToken })
        .end(function (err, hres) {
        if (err) {
            printDetailedError("Authentication failed", err);
            process.exit(1);
        }
        else {
            var cloudStudioHost = host === "https://na-east.jitterbit.com" ? "https://apps.na-east.jitterbit.com" : host;
            host = "https://citizen." + hres.body.ciDomain;
            var orgs = res.body.orgAttrs;
            orgs.forEach(function (org) {
                org["apiHost"] = hres.body.ciDomain;
            });
            var cres = {
                authenticationToken: authToken,
                defaultOrgId: orgId,
                orgs: orgs,
                cloudStudioHost: cloudStudioHost,
            };
            handleCitizenLoginResponse(cres, host, email, password);
        }
    });
}
exports.handleHarmonyLoginResponse = handleHarmonyLoginResponse;
function handleCitizenLoginResponse(res, host, email, password) {
    var authToken = res.authenticationToken;
    var defaultOrgId = res.defaultOrgId;
    var defaultEnvId = null;
    var orgs = res.orgs;
    var cloudStudioHost = res.cloudStudioHost;
    var count = 0;
    checkConfig();
    var original = readCred();
    var _loop_1 = function (org) {
        var reqURL = host + exports.citizenApiPrefix + "/orgs/" + org.orgId + "/environments";
        request.get(reqURL)
            .set("authToken", authToken)
            .end(function (erro, resp) {
            if (erro) {
                printDetailedError("Authentication failed", erro);
                writeCred(original);
                process.exit(1);
            }
            else {
                var envs = resp.body;
                org.envs = envs;
                org.defaultEnvId = envs.length === 1 ? envs[0].id : null;
                if (org.orgId === defaultOrgId) {
                    if (envs.length === 1) {
                        defaultEnvId = envs[0].id;
                    }
                    else {
                        printWarning("No default environment found. Set it with `jbcli config set envId=<id>`");
                    }
                }
                checkConfig();
                writeCred({
                    defaultEnvId: defaultEnvId,
                    defaultOrgId: defaultOrgId,
                    email: email,
                    host: host,
                    orgs: orgs,
                    password: base64.encode(password),
                    cloudStudioHost: cloudStudioHost,
                });
                if (++count === orgs.length) {
                    console.log("User " + email + " authenticated successfully into " + host + ".");
                }
            }
        });
    };
    for (var _i = 0, orgs_1 = orgs; _i < orgs_1.length; _i++) {
        var org = orgs_1[_i];
        _loop_1(org);
    }
}
exports.handleCitizenLoginResponse = handleCitizenLoginResponse;
function doTFA(host, email, password, deviceId, isHarmony) {
    if (isHarmony === void 0) { isHarmony = false; }
    console.log("Two-factor authentication is enabled for this organization. " +
        "Please check your email and input the code that you received from Jitterbit.");
    var code = "";
    try {
        code = prompt.hide("Code: ");
    }
    catch (err) {
        printDetailedError("Could not read code. This likely means that you are using Cygwin on Windows. " +
            "If so, please use Command Prompt instead.", err);
        process.exit(1);
    }
    var reqURL;
    var r;
    if (isHarmony) {
        reqURL = host + exports.harmonyApiPrefix + "/user/login/tfacode";
        r = request.put(reqURL)
            .send({ email: email, password: password, code: code, deviceId: deviceId });
    }
    else {
        reqURL = host + exports.citizenApiPrefix + "/login/tfacode";
        r = request.post(reqURL)
            .send({ email: email, password: password, code: code, deviceId: deviceId });
    }
    r.end(function (err, res) {
        if (err) {
            printDetailedError("Authentication failed", err);
            process.exit(1);
        }
        else {
            if (isHarmony && !res.body.status) {
                printDetailedError("Authentication failed", res.body);
                process.exit(1);
            }
            if (isHarmony) {
                handleHarmonyLoginResponse(res, host, email, password);
            }
            else {
                handleCitizenLoginResponse(res.body, host, email, password);
            }
        }
    });
}
exports.doTFA = doTFA;
function normalizePath(filepath) {
    if (!path.isAbsolute(filepath)) {
        filepath = path.normalize(path.join(process.cwd(), filepath));
    }
    return filepath;
}
exports.normalizePath = normalizePath;
function canWriteToPath(filepath) {
    if (fs.existsSync(filepath)) {
        console.log("The file at %s already exists.", filepath);
        return prompt("Overwrite it? (y/N) ") === "y";
    }
    return true;
}
exports.canWriteToPath = canWriteToPath;
function isLoggedIn() {
    checkConfig();
    return !!readCred()["password"];
}
exports.isLoggedIn = isLoggedIn;
function readCred() {
    var storedCred;
    try {
        storedCred = jsonfile.readFileSync(exports.credFile);
    }
    catch (err) {
        return {};
    }
    if (storedCred.password) {
        return storedCred;
    }
    else {
        return {};
    }
}
exports.readCred = readCred;
function writeCred(cred) {
    jsonfile.writeFileSync(exports.credFile, cred, { spaces: 2 });
}
exports.writeCred = writeCred;
function readConfig() {
    var storedConfig;
    var currentHost = readCred()["host"];
    try {
        storedConfig = jsonfile.readFileSync(exports.configFile)[currentHost] || {};
    }
    catch (err) {
        return {};
    }
    return storedConfig;
}
exports.readConfig = readConfig;
function writeConfig(config) {
    var storedConfig;
    var currentHost = readCred()["host"];
    try {
        storedConfig = jsonfile.readFileSync(exports.configFile);
    }
    catch (err) {
        storedConfig = {};
    }
    storedConfig[currentHost] = config;
    jsonfile.writeFileSync(exports.configFile, storedConfig, { spaces: 2 });
}
exports.writeConfig = writeConfig;
function checkConfig() {
    if (!fs.existsSync(exports.configDir)) {
        fs.mkdirSync(exports.configDir);
    }
    if (!fs.existsSync(exports.logDir)) {
        fs.mkdirSync(exports.logDir);
    }
    if (!fs.existsSync(exports.credFile)) {
        touch.sync(exports.credFile);
        fs.chmodSync(exports.credFile, "0600");
    }
    if (!fs.existsSync(exports.configFile)) {
        touch.sync(exports.configFile);
        fs.chmodSync(exports.configFile, "0600");
    }
}
exports.checkConfig = checkConfig;
function printDetailedError(info, e) {
    var err = "ERROR: " + info;
    var errCode = undefined;
    var errMessage = undefined;
    if (e) {
        if (typeof e === "string") {
            e = JSON.parse(e);
        }
        errCode = e.code || e.status || e.statusCode || e.errorCode || undefined;
        if (e.response) {
            if (typeof e.response === "string") {
                e.response = JSON.parse(e.response);
            }
            if (e.response.body) {
                if (typeof e.response.body === "string") {
                    e.response.body = JSON.parse(e.response.body);
                }
                if (e.response.body.errorMessage) {
                    if (typeof e.response.body.errorMessage === "string") {
                        try {
                            e.response.body.errorMessage = JSON.parse(e.response.body.errorMessage);
                        }
                        catch (jerr) {
                            errMessage = e.response.body.errorMessage;
                        }
                    }
                    if (e.response.body.errorMessage.errorMessage) {
                        errMessage = e.response.body.errorMessage.errorMessage;
                    }
                }
                else {
                    if (e.response.body.message) {
                        if (typeof e.response.body.message === "string") {
                            try {
                                e.response.body.message = JSON.parse(e.response.body.message);
                            }
                            catch (jerr) {
                                errMessage = e.response.body.message;
                            }
                        }
                        if (e.response.body.message.errorMessage) {
                            if (typeof e.response.body.message.errorMessage === "string") {
                                try {
                                    e.response.body.message.errorMessage = JSON.parse(e.response.body.message.errorMessage);
                                }
                                catch (jerr) {
                                    errMessage = e.response.body.message.errorMessage;
                                }
                            }
                            else {
                                errMessage = e.response.body.message.errorMessage.errorMessage;
                            }
                        }
                    }
                }
            }
        }
    }
    if (errCode && errMessage) {
        err += ": " + errMessage + ". (status " + errCode + ")";
    }
    else if (errCode) {
        err += ". (status " + errCode + ")";
    }
    else if (errMessage) {
        err += ": " + errMessage + ".";
    }
    else {
        err += ".";
    }
    console.error(err);
}
exports.printDetailedError = printDetailedError;
function printWarning(info) {
    var warn = "WARNING: " + info + ".";
    console.warn(warn);
}
exports.printWarning = printWarning;
function getOrgId() {
    return readConfig()["orgId"] || readCred()["defaultOrgId"];
}
exports.getOrgId = getOrgId;
function getEnvId() {
    return readConfig()["envId"] || readCred()["defaultEnvId"];
}
exports.getEnvId = getEnvId;
function setRecipeId(recipePath, id) {
    var recipe;
    try {
        recipe = jsonfile.readFileSync(recipePath);
    }
    catch (err) {
        console.error("ERROR: Could not read recipe file.");
        process.exit(1);
    }
    recipe["id"] = id;
    jsonfile.writeFileSync(recipePath, recipe, { spaces: 2 });
}
exports.setRecipeId = setRecipeId;
function extractProjectDetails(jpk) {
    var prjDetails = {
        id: null,
        name: null,
        projectVariables: [],
    };
    var prjVarFiles = [];
    var zip = new AdmZip(jpk);
    zip.getEntries().forEach(function (entry) {
        var entryName = entry.entryName;
        if (entryName.includes("ProjectVariable")) {
            prjVarFiles.push(entryName);
        }
        else if (entryName.endsWith("/jitterpak.properties")) {
            //      console.log(zip.readAsText(entryName));
            var propsReader = new PropertiesReader(null).read(zip.readAsText(entryName));
            prjDetails.id = propsReader.get("project_id");
            prjDetails.name = propsReader.get("project_name");
        }
    });
    var DOMParser = new xmldom.DOMParser({
        errorHandler: {
            error: function (e) {
                printDetailedError("There was an error parsing the JPK for project variables", e);
                process.exit(1);
            },
            fatalError: function (e) {
                printDetailedError("There was an error parsing the JPK for project variables", e);
                process.exit(1);
            },
            warning: function (w) { return printWarning("There was a problem parsing the JPK for project variables: " + w); },
        },
        locator: {},
    });
    prjVarFiles.forEach(function (prjVarFile) {
        //    console.log(zip.readAsText(prjVarFile));
        var prjVarDom = DOMParser.parseFromString(zip.readAsText(prjVarFile), "text/xml");
        var header = prjVarDom.documentElement.getElementsByTagName("Header").item(0);
        if (header.getAttribute("Deleted") === "true") {
            return;
        }
        var prjVar = {
            displayName: null,
            name: header.getAttribute("Name"),
        };
        var properties = prjVarDom.documentElement.getElementsByTagName("Item");
        for (var i = 0; i < properties.length; i++) {
            var property = properties.item(i);
            var key = property.getAttribute("key");
            var value = property.getAttribute("value");
            switch (key.toLowerCase()) {
                case "label":
                    prjVar.displayName = value;
                    break;
                case "description":
                    prjVar.description = value;
                    break;
                case "hide_value":
                    if (value === "1") {
                        prjVar.type = "password";
                    }
                    break;
                case "value":
                    prjVar.value = value;
                    break;
                default:
                    break;
            }
        }
        prjDetails.projectVariables.push(prjVar);
    });
    prjDetails.projectVariables.sort(function (a, b) {
        if (a.name < b.name) {
            return -1;
        }
        else if (a.name > b.name) {
            return 1;
        }
        else {
            return 0;
        }
    });
    return prjDetails;
}
exports.extractProjectDetails = extractProjectDetails;
function cloudStudioLogin(host, email, password) {
    var url = host + exports.csApiPrefix + "/user/login";
    request.post(url)
        .send({ email: email, password: password })
        .end(function (erro, res) {
        if (erro) {
            printDetailedError("Unable to login to Cloud Studio", erro);
        }
        else {
            var authToken = res.header["x-jitterbit-authorizationtoken"];
            var cred = readCred();
            cred["csAuthToken"] = authToken;
            writeCred(cred);
        }
    });
}
exports.cloudStudioLogin = cloudStudioLogin;
