"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var async = require("async");
var request = require("superagent");
var common_1 = require("../common");
function deployRecipe(args) {
    common_1.authenticate(args, _deployRecipe);
}
exports.deployRecipe = deployRecipe;
function _deployRecipe(args, host, token) {
    var id = args["id"];
    var cfgFile = common_1.normalizePath(args["cfg"]);
    var cfg;
    try {
        cfg = require(cfgFile);
    }
    catch (e) {
        common_1.printDetailedError("Failed to read configure file", e);
        process.exit(1);
    }
    var reqURL = host + common_1.citizenApiPrefix + "/orgs/" + common_1.getOrgId() + "/recipes/" + id;
    request.get(reqURL)
        .set("authToken", token)
        .end(function (err, res) {
        if (err) {
            common_1.printDetailedError("Could not get recipe with ID " + id, err);
            process.exit(1);
        }
        else {
            var recipe_1 = res.body;
            for (var _i = 0, _a = recipe_1.projectVariables; _i < _a.length; _i++) {
                var rpv = _a[_i];
                rpv.value = cfg[rpv.name] || "";
            }
            var actionFuncs = [];
            for (var _b = 0, _c = recipe_1.steps; _b < _c.length; _b++) {
                var step = _c[_b];
                if (step.type === "org.jitterbit.integration.data.entity.RunOperation" && !step.hidden) {
                    var newFunc = _makeActionFunc("run-operation", host, token, recipe_1, step, step);
                    if (newFunc) {
                        actionFuncs.push(newFunc);
                    }
                }
                else if (step.projectVariables && step.projectVariables.action) {
                    var action = step.projectVariables.action;
                    var stepForRequest = _findStepByPath(recipe_1, action.useStepAsRequest);
                    if (!stepForRequest) {
                        continue;
                    }
                    var newFunc = _makeActionFunc(action.name, host, token, recipe_1, step, stepForRequest);
                    if (newFunc) {
                        actionFuncs.push(newFunc);
                    }
                }
            }
            async.series(actionFuncs, function (serr, sres) {
                if (serr) {
                    common_1.printDetailedError("Action failed", serr);
                    process.exit(1);
                }
                else {
                    console.log("All actions succeeded. Now saving recipe, please wait...");
                    reqURL = host + common_1.citizenApiPrefix + "/orgs/" + common_1.getOrgId() + "/environments/" + common_1.getEnvId() + "/recipes";
                    request.post(reqURL)
                        .set("authToken", token)
                        .set("x-jitterbit-run-operations", "TRUE")
                        .send(recipe_1)
                        .end(function (erro, resp) {
                        if (erro) {
                            common_1.printDetailedError("Failed to deploy recipe", erro);
                            process.exit(1);
                        }
                        else {
                            console.log("Recipe deployed, ID %s.", resp.body.deployedRecipeId);
                        }
                    });
                }
            });
        }
    });
}
function _findStepByPath(recipe, path) {
    for (var _i = 0, _a = recipe.steps; _i < _a.length; _i++) {
        var step = _a[_i];
        if (step.path === path) {
            return step;
        }
    }
    return null;
}
function _fillStep(step) {
    for (var _i = 0, _a = step.properties; _i < _a.length; _i++) {
        var property = _a[_i];
        property.value = property.defaultValue;
    }
    return step;
}
function _lookupProperty(propName, step) {
    for (var _i = 0, _a = step.properties; _i < _a.length; _i++) {
        var property = _a[_i];
        if (property.name === propName) {
            return property;
        }
    }
    return null;
}
var ITERATIONS = 20;
var INTERVAL = 3000;
function _makeActionFunc(type, host, token, recipe, step, stepForRequest) {
    switch (type) {
        case "test-endpoint-connection":
            stepForRequest = _fillStep(stepForRequest);
            return function (callback) {
                process.stdout.write("Running connection test for step '" + step.name + "'... ");
                request.put(host + common_1.citizenApiPrefix + "/orgs/" + common_1.getOrgId() + "/environments/" + common_1.getEnvId() +
                    "/discovery/" + stepForRequest.type + "/test-endpoint-connection")
                    .set("authToken", token)
                    .set("Content-Type", "application/json")
                    .send({
                    recipe: recipe,
                    step: stepForRequest,
                })
                    .end(function (err, res) {
                    if (err) {
                        console.log("Failed!");
                    }
                    else {
                        console.log("Success!");
                    }
                    callback(err, res);
                });
            };
        case "run-operation":
            var operationProp = _lookupProperty("operationName", stepForRequest);
            if (!operationProp) {
                return null;
            }
            var operationName_1 = operationProp.value || operationProp.defaultValue;
            if (!operationName_1) {
                return null;
            }
            return function (callback) {
                process.stdout.write("Running intermediate operation for step '" + step.name + "'... ");
                request.post(host + common_1.citizenApiPrefix + "/orgs/" + common_1.getOrgId() + "/environments/" + common_1.getEnvId() +
                    "/operations/" + operationName_1)
                    .set("authToken", token)
                    .set("Content-Type", "application/json")
                    .send(recipe)
                    .end(function (err, res) {
                    if (err) {
                        console.log("Failed!");
                        callback(err, res);
                    }
                    else {
                        var operationInstanceGuid_1 = res.body["opInstanceGuid"];
                        var operationGuid_1 = res.body["operationGuid"];
                        async.retry({ times: ITERATIONS, interval: INTERVAL }, function (rcallback, results) {
                            _checkOpLog(host, token, operationInstanceGuid_1, operationGuid_1, rcallback);
                        }, function (rerr, rres) {
                            if (rerr) {
                                console.log("Failed!");
                                callback(rerr, rres);
                            }
                            else {
                                if (rres.error) {
                                    console.log("Failed!");
                                    callback(rres, null);
                                }
                                else {
                                    console.log("Success!");
                                    callback(null, rres);
                                }
                            }
                        });
                    }
                });
            };
        case "submit-http-form":
            stepForRequest = _fillStep(stepForRequest);
            return function (callback) {
                process.stdout.write("Running HTTP test connection for step '" + step.name + "'... ");
                request.put(host + common_1.citizenApiPrefix + "/orgs/" + common_1.getOrgId() + "/environments/" + common_1.getEnvId() +
                    "/discovery/http-form")
                    .set("authToken", token)
                    .set("Content-Type", "application/json")
                    .send({
                    recipe: recipe,
                    step: stepForRequest,
                })
                    .end(function (err, res) {
                    if (err) {
                        console.log("Failed!");
                    }
                    else {
                        console.log("Success!");
                    }
                    callback(err, res);
                });
            };
        default:
            break;
    }
}
function _checkOpLog(host, token, operationInstanceGuid, operationGuid, callback) {
    var reqURL = host + common_1.citizenApiPrefix + "/orgs/" + common_1.getOrgId() + "/environments/" + common_1.getEnvId() +
        "/operation-logs?operationGuid=" + operationGuid + "&operationInstanceId=" + operationInstanceGuid;
    request.get(reqURL)
        .set("authToken", token)
        .set("Content-Type", "application/json")
        .end(function (err, res) {
        var opLog = res.body;
        if (opLog.errorCode) {
            callback(null, { error: true, errorMessage: opLog.errorMessage });
        }
        else {
            var status_1 = _evaluateOpLogStatus(opLog);
            switch (status_1) {
                case 0 /* NOT_FINISHED */:
                    callback("not finished", null);
                    break;
                case 2 /* ERROR */:
                    callback(null, { error: true, errorMessage: _extractOpLogMessage(opLog) });
                    break;
                case 1 /* SUCCESS */:
                    callback(null, { error: false, errorMessage: null });
                    break;
                default:
                    callback("default", null);
                    break;
            }
        }
    });
}
function _evaluateOpLogStatus(opLog) {
    if (!opLog.records || opLog.records.length === 0) {
        // try again
        return 0 /* NOT_FINISHED */;
    }
    var opStatusId = opLog.records[0].attributes.operationStatusId;
    switch (opStatusId) {
        case 0:
        case 1:
        case 2:
        case 3:
        case 10:
            // retry
            return 0 /* NOT_FINISHED */;
        case 4:
        case 5:
        case 6:
            // success
            return 1 /* SUCCESS */;
        default:
            // error
            return 2 /* ERROR */;
    }
}
function _extractOpLogMessage(opLog) {
    if (!opLog.records || opLog.records.length === 0) {
        // try again
        return "No message";
    }
    return opLog.records[0].attributes.message;
}
